package arzumify.polyenergy.util;

import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.polyenergy.impl.CoordinateMatchMaker;
import arzumify.polyenergy.impl.ProviderDetails;
import arzumify.polyenergy.impl.ReceiverDetails;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerBlockEntityEvents;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;

import java.util.ArrayList;

// Cables can have ONE provider and MANY receivers.
@SuppressWarnings("unused")
public class SimpleCable extends BlockEntity implements EnergyProvider, EnergyReceiver, ServerBlockEntityEvents.Unload, ServerBlockEntityEvents.Load {
    private final long capacity;
    private final long inputRate;
    private final long outputRate;
    private final ProviderDetails providerDetails = ProviderDetails.NewSimple(pos, this);
    private final ReceiverDetails receiverDetails = ReceiverDetails.NewSimple(pos, this);
    private EnergyProvider currentProvider;
    private long energy = 0;

    public SimpleCable(BlockEntityType<?> type, BlockPos pos, BlockState state, long capacity, long inputRate, long outputRate) {
        super(type, pos, state);
        this.capacity = capacity;
        this.inputRate = inputRate;
        this.outputRate = outputRate;
        CoordinateMatchMaker.addReceiver(receiverDetails);
    }

    public static void tick(World ignoredWorld, BlockPos ignoredPos, BlockState ignoredState, SimpleCable cable) {
        if (cable.currentProvider != null && cable.energy < cable.capacity) {
            cable.energy += cable.currentProvider.extract(Math.min(cable.capacity - cable.energy, cable.inputRate), cable);
        }
    }

    @Override
    public long extract(long amount, EnergyReceiver receiver) {
        System.out.println("Extracting " + amount + " energy");
        long extracted = Math.min(Math.min(outputRate, amount), energy);
        energy -= extracted;
        return extracted;
    }

    @Override
    public void exists(EnergyReceiver receiver, ArrayList<Vec3i> pointsOfPresence) {
        if (currentProvider != null) {
            receiver.ready(this);
        }
    }

    @Override
    public void ready(EnergyProvider provider) {
        if (currentProvider == null) {
            currentProvider = provider;
            CoordinateMatchMaker.addProvider(providerDetails);
        }
    }

    @Override
    public void unready(EnergyProvider provider) {
        if (currentProvider == provider) {
            currentProvider = null;
            CoordinateMatchMaker.removeProvider(providerDetails);
        }
    }

    @Override
    public void onLoad(BlockEntity blockEntity, ServerWorld serverWorld) {
        CoordinateMatchMaker.addReceiver(receiverDetails);
    }

    @Override
    public void onUnload(BlockEntity blockEntity, ServerWorld serverWorld) {
        CoordinateMatchMaker.removeProvider(providerDetails);
        CoordinateMatchMaker.removeReceiver(receiverDetails);
        currentProvider = null;
    }
}
